@@ -26,9 +26,6 @@
 
 #include "exchange-mapi-account-listener.h"
 
-/* This definition should be in-sync with the definition in camel-mapi-store.c */
-#define EXCHANGE_MAPI_PASSWORD_COMPONENT "ExchangeMAPI"
-
 #define MAPI_URI_PREFIX   "mapi://" 
 #define MAPI_PREFIX_LENGTH 7
 
