@@ -159,7 +159,7 @@ fetch_items_summary_cb (FetchItemsCallbackData *item_data, gpointer data)
 	MailItem *item = g_new0(MailItem , 1);
 
 	if (camel_debug_start("mapi:folder")) {
-		exchange_mapi_debug_property_dump (item_data->properties);
+		exchange_mapi_debug_dump_properties (item_data->conn, item_data->fid, item_data->properties);
 		camel_debug_end();
 	}
 
@@ -1707,6 +1707,12 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 	GSList *src_msg_ids = NULL;
 	gboolean success = TRUE;
 
+	if (CAMEL_IS_MAPI_FOLDER (source)) {
+		/* make sure changed flags are written into the server */
+		if (!mapi_folder_synchronize_sync (source, FALSE, cancellable, error))
+			return FALSE;
+	}
+
 	if (!CAMEL_IS_MAPI_FOLDER (source) || !CAMEL_IS_MAPI_FOLDER (destination) ||
 	    (CAMEL_MAPI_FOLDER (source)->type & CAMEL_MAPI_FOLDER_PUBLIC) != 0 ||
 	    (CAMEL_MAPI_FOLDER (destination)->type & CAMEL_MAPI_FOLDER_PUBLIC) != 0) {
